<?php

namespace Drupal\spotify_playing\Controller;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\spotify_playing\SpotifyHelper;
use SpotifyWebAPI\SpotifyWebAPI;
use SpotifyWebAPI\Session;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Url;

/**
 * Handles Authorization for Spotify API.
 */
class SpotifyAuthController extends ControllerBase {

  /**
   * Settings key.
   */
  const SETTINGS = 'spotify_playing.settings';

  /**
   * Spotify Helper service.
   *
   * @var \Drupal\spotify_playing\SpotifyHelper
   */
  protected $spotify;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('spotify_playing.spotify_helper')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function __construct(SpotifyHelper $spotify) {
    $this->spotify = $spotify;
  }

  /**
   * Validate and dump API/Session information.
   */
  public function content() {

    $config = \Drupal::configFactory()->getEditable(static::SETTINGS);

    $client_id = $config->get('client_id');
    $client_secret = $config->get('client_secret');

    $api_object = \Drupal::service('spotify_playing.spotify_helper')
      ->api();

    $dump = [
      'client_id'     => $client_id,
      'client_secret' => $client_secret,
      'access_token'  => $api_object['session']->getAccessToken(),
      'refresh_token' => $api_object['session']->getRefreshToken(),
    ];

    $response = new CacheableJsonResponse($dump);

    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($dump));

    return $response;

  }

  /**
   * Authorization Callback route for API.
   */
  public function authorize() {

    $current_request = \Drupal::request();

    $config = \Drupal::configFactory()->getEditable(static::SETTINGS);

    $client_id = $config->get('client_id');
    $client_secret = $config->get('client_secret');

    $form_uri = Url::fromRoute('spotify_playing.settings')->toString();
    $redirect_uri = Url::fromRoute('spotify_playing.authorization', [], [
      'absolute' => TRUE,
    ])->toString();

    if (!empty($client_id) && !empty($client_secret)) {
      $code = $current_request->query->get('code');

      $session = new Session(
      $client_id,
      $client_secret,
      $redirect_uri
      );

      $api = new SpotifyWebAPI([
        'auto_refresh' => TRUE,
      ]);

      if (!empty($code)) {
        try {
          $session->requestAccessToken($_GET['code']);
          $api->setAccessToken($session->getAccessToken());

          $config
            ->set('access_token', $session->getAccessToken())
            ->set('refresh_token', $session->getRefreshToken())
            ->save();

          header('Location: ' . $redirect_uri);
        }
        catch (Exception $e) {
          echo 'Spotify API Error: ' . $e->getCode();
          die();
        }
      }

      $accessToken = $config->get('access_token');

      if (empty($accessToken)) {
        $config
          ->set('access_token', '')
          ->set('refresh_token', '')
          ->save();

        $authorizeUrlOptions = [
          'scope' => [
            'user-read-currently-playing',
            'user-read-recently-played',
            'user-read-playback-state',
            'user-modify-playback-state',
          ],
        ];

        header('Location: ' . $session->getAuthorizeUrl($authorizeUrlOptions));
        die();
      }

    }

    header('Location: ' . $form_uri);
    die();
  }

}
